/*
 * Copyright (C) by LeftHand sp. z o.o.
 * See http://www.corporateoffice.pl
 */

#include "httpd.h"
#include "http_config.h"
#include "http_main.h"
#include "http_protocol.h"

#include "co_context.h"
#include "co_output.h"

/* 
 * Sends str to client after HTML tag. If used as the first cou_puts* call,
 * sends header with "text/html" MIME type.
 * If cou_putsHTML is used for the first time, it sends <HTML> tag.
 */
void
cou_putsHTML( co_context *ctx, const char* str)
{
    request_rec *r;

    r = ctx->request;
    /* check if HTML is expected; prepare client for HTML */
    switch (ctx->html_mode) {
        case CO_NO_HTML :
        case CO_FINISHED: 
			return; break;
        case CO_BEFORE_HTML:
            r->content_type = "text/html";
            ap_send_http_header(r);
            ap_rputs(DOCTYPE_HTML_4_0T, r);
            ap_rputs("<HTML>\n", r);
            ctx->html_mode = CO_HTML;
            break;
        case CO_TEXT   :
            ap_rputs("<HTML>\n", r);
            //ctx->html_mode = CO_HTML;
            break;
        case CO_HTML   :
        case CO_HEAD   :
        case CO_BODY   : break;
        case CO_HEAD_SCRIPT :
        case CO_BODY_SCRIPT : return; break;
    }
    ap_rputs( str, r );
}

/* 
 * Sends str to client. If used as the first cou_puts* call,
 * sends header with "text/plain" MIME type.
 */
void
cou_putsTEXT( co_context *ctx, const char* str)
{
    request_rec *r;

    r = ctx->request;
    /* check if output is expected; prepare client for text */
    switch (ctx->html_mode) {
        case CO_NO_HTML :
        case CO_FINISHED: 
			return; break;
        case CO_BEFORE_HTML:
            r->content_type = "text/plain";
            ap_send_http_header(r);
            ctx->html_mode = CO_TEXT;
            break;
        case CO_TEXT   : 
        case CO_HTML   : 
        case CO_HEAD   : 
        case CO_BODY   : 
        case CO_HEAD_SCRIPT : 
        case CO_BODY_SCRIPT : break;
    }
    ap_rputs( str, r );
}

/* 
 * Sends finishing string to client. 
 * If initial state was different from NO_HTML and any
 * of the cou_puts* were used, it closes tags send by cou_puts*.
 */
void
cou_finishHTML( co_context *ctx )
{
    request_rec *r;

    r = ctx->request;
    /* check state of HTML output */
    switch (ctx->html_mode) {
    case CO_NO_HTML     :
    case CO_FINISHED    :
    case CO_BEFORE_HTML :
    case CO_TEXT:
        return;
        break;
    case CO_BODY_SCRIPT : 
		ap_rputs("</SCRIPT>\n</BODY>\n</HTML>\n", r);
        break;
    case CO_HEAD_SCRIPT : ap_rputs("</SCRIPT>\n", r);
    case CO_HEAD   : ap_rputs("</HEAD>\n<BODY>\n", r);
    case CO_BODY   : ap_rputs("</BODY>\n", r);
    case CO_HTML   : ap_rputs("</HTML>\n", r);
        ctx->html_mode = CO_FINISHED;
        break;
    }
}
