#include "types.t"      
#include "defines.t"      

// CLASS CATALOG

class catalog_root
{ 
  
  doc "in this class we keep information about roots for roles and catalogs";
  
  area "sys";

#include "check_acl.t"
  
  inherits generic_acl;
  
  container_class 'role';
  container_column_name 'id_role';
      
 // PROPERTIES
      
#include "generic_acl_props.t"      

  prop id_role
  {      
    type CO_INTEGER;      
    default 0;      
    params '';      
    representant 1;      
    foreign 'role';      
    foreign_prop 'id';      
    position 200;    
  };    

  prop class_name
  {      
    type CO_STRING;      
    default "''";      
    params '';      
    representant 1;      
    foreign 'meta_class';      
    foreign_prop 'class_name';      
    position 200;    
  };    
   
  prop root_name
  {      
    type CO_STRING;      
    default "''";      
    params '';      
    representant 1;      
    foreign '';      
    foreign_prop '';      
    position 200;    
  };    
 
  prop id_root
  {
    type CO_INTEGER;
    default 0;
    params '';
    representant 1;
    foreign '';
    foreign_prop '';
    position 300;
  };
 
 method getRootId
  {
     doc "this method is used to get id_root for given catalog type and current user";
     doc "Returns Integer.";

     id_permission __PERMISSION_SELECT;
     sec_type __METHOD_SEC_TYPE_CLASS;
 
     __PARAMS_HEADER_WITH_CALLBACK
     param class_name
     {
       doc "this is the name of the base class for catalog for which you ask to get id_root";
       type CO_STRING;
     };
 
     declare language_c
     {
        C "int id_root;";
     };
 
     body language_c
     {
       C
       %%
       __SQL_GET_INTEGER(id_root,"SELECT id_root FROM co_%s_%s WHERE class_name = '%s' AND id_role = %d",
                       par_area, par_class, class_name, cnx->id_role);
       __CREATE_JS_FROM_INTEGER(id_root);
       __RETURN_INTEGER(id_root);
       %%
       ;
     };
  };

 method getRootIdAndName
  {
     doc "this method is used to get id_root and root_name for given catalog type and current user";
     doc "Returns recordset.";

     id_permission __PERMISSION_SELECT;
     sec_type __METHOD_SEC_TYPE_CLASS;
 
     __PARAMS_HEADER_WITH_CALLBACK
     param class_name
     {
       doc "this is the name of the base class for catalog for which you ask to get id_root";
       type CO_STRING;
     };
 
     body language_c
     {
       C
       %%
		__CREATE_QUERY("SELECT id_root, root_name FROM co_%s_%s WHERE class_name = '%s' AND id_role = %d",
						par_area, par_class, class_name, cnx->id_role);
		__GET_RESULT_AND_CREATE_JS
       %%
       ;
     };
  };
 
};

//END OF CLASS CATALOG_ROOT