#include "types.t"
#include "defines.t"

class meta_tree
{
	area sys;

	doc
	%%
	This class describes relations belonging to every cross-class tree.
	Instead of creating wrappers like in generic_catalog based classes,
	you describe relations parent-child.
	Every pair "parent class - child class" is unique for given tree name.
	%%
	;

#include "check_acl.t"
   
   container_class '';
   container_column_name '';

#include "generic_acl_props.t"    

   prop tree_name
   {
     type CO_STRING;
     default "''";
     params '';
     representant 1;
     foreign '';
     foreign_prop '';
     position 200;
   };
   
   prop parent_class_name
   {
     type CO_STRING;
     default "''";
     params '';
     representant 0;
     foreign 'meta_class';
     foreign_prop 'class_name';
     position 300;
   };

   prop parent_prop_name
   {
     type CO_STRING;
     default "''";
     params '';
     representant 0;
     foreign '';
     foreign_prop '';
     position 400;
   };
   
   prop child_class_name
   {
     type CO_STRING;
     default "''";
     params '';
     representant 0;
     foreign 'meta_class';
     foreign_prop 'class_name';
     position 500;
   };

   prop child_prop_name
   {
     type CO_STRING;
     default "''";
     params '';
     representant 0;
     foreign '';
     foreign_prop '';
     position 600;
   };

   prop unique_child
   {
     doc "Specifies if parent has unique child from given child class; e.g. user has only 1 start_page.";
     type CO_INTEGER;
     default 0;
     params 'VALUES=new Array(0,1);';
     representant 0;
     foreign '';
     foreign_prop '';
     position 700;
   };

            
   inherits generic_acl;

   index in1
   {
    unique 0;
    column tree_name;
    column parent_class_name;
    column child_class_name;
   };

	method getChildren
	{
		doc "This method reads all children from given tree for given parent. Pass -1 as par_id. Returns recordset and JavaScript."
		{
			pl "Ta metoda znajduje wszystkie dzieci z danego drzewa i dla danego ojca. Podaj -1 jako par_id. Zwraca rekordset i JavaScript.";
		};
		doc "Real security should be based on parent_class_name and parent_id (GET_RELATED).";

		id_permission __PERMISSION_SELECT;
		sec_type __METHOD_SEC_TYPE_CLASS;
   
		__PARAMS_HEADER_WITH_CALLBACK

		param tree_name
		{
			type CO_STRING;
			doc "Name of the tree"
			{
				pl "Nazwa drzewa";
			};
		};

		param parent_class_name
		{
			type CO_STRING;
			doc "Name of class of the parent"
			{
				pl "Nazwa klasy ojca";
			};
		};

		param parent_id
		{
			type CO_INTEGER;
			doc "id of the parent"
			{
				pl "id ojca";
			};
		};

		declare language_c
		{
			C 'const char * parent_prop_name;';
			C 'const char * parent_prop_val="";';
			C 'const char * child_class_name;';
			C 'const char * child_prop_name;';
			C 'co_recordset * out_corec;';
			C 'co_recordset * classes_corec;';
			C 'int nTuples=0;';
			C 'int n=0;';
		};

		body language_c
		{
			C 
			%%
			// read child classes and their props; build query and get children
			// get all child classes
			__CREATE_QUERY(
"SELECT child_class_name FROM co_%s_%s WHERE tree_name='%s' AND parent_class_name='%s'",
				par_area, par_class, tree_name, parent_class_name)
			__SQL_GET_VRESULT(classes_corec)
			// keep it!
			nTuples = cor_GetTupleCount(cnx, classes_corec);
			out_corec = cor_CreateEmpty(cnx);
			// for each child class name
			for (n=0; n<nTuples; n++) {
				child_class_name = cor_GetValue(cnx, classes_corec, 
					n, 0);
				// get parent (related) prop name and child prop name
				__CREATE_QUERY(
					"SELECT parent_prop_name, child_prop_name FROM co_%s_%s "
					"WHERE tree_name='%s' AND parent_class_name='%s' "
					"AND child_class_name='%s'",
					par_area, par_class, 
					tree_name, parent_class_name,
					child_class_name)
				__SQL_GET_RESULT
				parent_prop_name = cor_GetCopyOfValue(cnx, corec, 
					0, 0);
				child_prop_name = cor_GetCopyOfValue(cnx, corec, 
					0, 1);
				__CLEAR_RESULT
				if (*parent_prop_name) // not empty - select children
				{
					// get parent related prop value - just getProp
					__SQL_GET_STRING(parent_prop_val,
						"SELECT \"%s\" FROM co_%s_%s WHERE id=%d",
						parent_prop_name, par_area, parent_class_name, parent_id)
					// get related children from class: child_class_name
					__CREATE_QUERY(
"SELECT id, '%s' as class, name FROM cov_%s_%s WHERE \"%s\"='%s' ORDER BY name",
					child_class_name, par_area, child_class_name, child_prop_name, parent_prop_val)
				}
				else // empty parent_prop_name: get all children from their class
				{
					__CREATE_QUERY(
"SELECT id, '%s' as class, name FROM cov_%s_%s ORDER BY name",
					child_class_name, par_area, child_class_name)
				}
				/* too much copying: */
				__SQL_GET_RESULT
				cor_AppendRecordset(cnx, out_corec, corec);
				__CLEAR_RESULT
				/* Should be more efficient, but Why out_corec is empty?
				__SQL_APPEND_VRESULT(out_corec)
				*/
			} // for each child_class_name
			cor_Clear(cnx, classes_corec);
			if (!cor_IsEmpty(cnx, out_corec)) {
				__CREATE_JS_FROM(out_corec)
			}
			__RETURN_RECORDSET(out_corec)

			%%
			; 
		};  

	}; // method commitChange


};  
